/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */

package jogo;

import java.awt.Graphics2D;
import java.awt.Image;
import javax.swing.ImageIcon;

/**
 * 
 * @author Emerson Ribeiro de Mello
 */
public abstract class Elemento{

    protected JogoTimer jogo;  // referencia a area de Desenho
    protected Image icone; // imagem 
    protected String imagemNome; // nome da imagem 
    protected int largura; // largura da imagem
    protected int altura;  // altura da imagem
    protected int posX;    // coordenada X no Canvas
    protected int posY;    // coordenada Y no Canvas
    protected int matColuna; // posicao na matrix 17x11
    protected int matLinha;  // posicao na matrix 17x11
    protected int velocidade; // indica a taxa de deslocamento em pixel

   /**
     * 
     * @param p
     * @param imagemNome
     * @param mx
     * @param my 
     */
    public Elemento(JogoTimer p, String imagemNome, int mx, int my) {
        this.jogo = p;
        this.imagemNome = imagemNome;
        this.icone = this.carregarImagem(this.imagemNome);
        this.largura = this.icone.getWidth(null);
        this.altura = this.icone.getHeight(null);

        this.posX = mx * jogo.larguraBloco;
        this.posY = my * jogo.larguraBloco;
        this.matColuna = mx;
        this.matLinha = my;

        this.velocidade = 40;


    }

    /**
     * Para atualizar as coordenadas e outros atributos do elemento
     */
    public abstract void atualizar();

    /**
     * Para desenhar o Elemento na tela
     * @param g contexto gráfico
     */
    public abstract void desenhar(Graphics2D g);

    /**
     * Para carregar a imagem do disco. Esse método não pode ser sobreescrito. 
     * Veja a palavra final.
     * @param imagem Nome da imagem. Ex: bomba.png
     * @return Retorna um objeto Image
     */
    public final Image carregarImagem(String imagem) {
        ImageIcon iicon = new ImageIcon(this.getClass().getResource("/imagens/" + imagem));
        return iicon.getImage();
    }
    
    
    
}
