/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import principal.JavaBomber;
import java.awt.event.ActionEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.Timer;

/**
 * Essa clase contém a lógica principal do Jogo. A atualização da tela é feita
 * através de o Timer do pacote Swing. Por este motivo, essa classe tem que
 * implementar a Interface ActionListener
 * Essa classe herda de Canvas, pois a classe Canvas é ideal para fazer desenhos
 * e permite usufrir o bufferStrategy para melhorar a taxa de atualização da tela
 * 
 * @author Emerson Ribeiro de Mello
 */
public class JogoTimer extends Canvas implements ActionListener {

    private int tempobomba;
    public JavaBomber framePrincipal; // frame principal do jogo
    public Jogador jog1; // Jogador1
    public Jogador jog2; // Jogador2
    public Timer timer; // Timer responsável pela atualização do jogo
    public int larguraBloco; // Dimensao dos personabens, fundos. É 40
    public int colunas; // total de colunas na matriz do jogo
    public int linhas; // total de linhas na matriz do jogo
    private Image imgcenarios[]; // contém as imagens de fundo do cenário
    protected ArrayList<bomba> arraybomba = new ArrayList<bomba>();
    private bomba bombateste;
    private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
    private int NSb;
    private int NSv;
    private int NSvi;
    // 0 = ferro
    // 1 = tijolo
    // 2 = vazio
    // 3 = bomba
    // 4 = vida
    // 5 = 
    // 6 = 
    // 7 = 
    // 8 = 
    // 9 = 
    // ...
    public short[][] cenario = { //11x17 (linha,coluna)
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 2, 2, 0},
        {0, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 2, 0},
        {0, 2, 2, 1, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 0},
        {0, 2, 0, 1, 0, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 2, 0},
        {0, 2, 2, 1, 2, 2, 2, 1, 1, 1, 2, 1, 1, 1, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    };

    //int numero = 1+(int)(Math.random() * 2);
    public JogoTimer(JavaBomber p) {
        super();
        this.NSb = 0;
        this.NSv = 0;
        this.NSvi = 0;
        this.framePrincipal = p;
        this.setBackground(Color.GRAY);
        this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
        this.larguraBloco = 40;
        this.colunas = 17;
        this.linhas = 11;
        this.taxaDeAtualizacao = 120;
        this.imgcenarios = new Image[7];
        this.imgcenarios[0] = (new ImageIcon(this.getClass().getResource("/imagens/ferro.png"))).getImage();
        this.imgcenarios[1] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[2] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[3] = (new ImageIcon(this.getClass().getResource("/imagens/explobonus.png"))).getImage();
        this.imgcenarios[4] = (new ImageIcon(this.getClass().getResource("/imagens/bonusbomba.png"))).getImage();
        this.imgcenarios[5] = (new ImageIcon(this.getClass().getResource("/imagens/bonusvida.png"))).getImage();
       
    }
/**
     * 
     * @return retorna cenario aleatorio
     */
    public final short[][] getCenario() {

        
        short[][] cenario1 = { //11x17 (linha,coluna)
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 2, 2, 0},
        {0, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 2, 0},
        {0, 2, 2, 1, 1, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 0},
        {0, 2, 0, 1, 0, 2, 0, 1, 0, 1, 0, 1, 0, 1, 0, 2, 0},
        {0, 2, 2, 1, 2, 2, 2, 1, 1, 1, 2, 1, 1, 1, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    };

        
        int a = 0;
        int b = 0;
        int cX = 3;
        int cY = 1;
        int i = 1;
        while (a < 104) {
            a++;
            if (cX > 15) {
                cX = 1;
                cY++;
                if (i == 1) {
                    i = 2;
                } else {
                    i = 1;
                }
            }
            int numero = 1 + (int) (Math.random() * 2);
            if (numero == 1) {
                if (b < 2) {
                    cenario1[cY][cX] = 2;
                    b++;
                    cX = cX + i;
                } else {
                    cenario1[cY][cX] = 1;
                    cX = cX + i;
                }
            }
            if (numero == 2) {
                cenario1[cY][cX] = 1;
                cX = cX + i;
            }
            if (a % 10 == 0) {
                b = 0;
            }
        }
        cenario1[2][1] = 2;
        cenario1[1][2] = 2;
        cenario1[1][1] = 2;
        cenario1[9][14] = 2;
        cenario1[8][15] = 2;
        cenario1[9][15] = 2;
        return cenario1;
    }

    /**
     * Método invocado para iniciar a partida
     */
    public void comecaOJogo() {
        // criando o Timer com taxa de atualização de N milisegundos
        if (this.timer != null) {
            this.timer.stop();
        }
         this.cenario = this.getCenario();
        this.timer = new Timer(taxaDeAtualizacao, this);
        // criando 2 buffer para acelerar a taxa de desenho
        createBufferStrategy(2);

        // KeyEvent possui codigos para as teclas. Por exemplo, para obter
        // o codigo da tecla 'seta para cima' basta invocar KeyEvent.VK_UP
        // Veja o uso do KeyEvent na criacao dos jogadores abaixo

        // criando Jogador 1
        this.jog1 = new Jogador(this, "vermelho.png", "bomba1.png", 1, 1,
                KeyEvent.VK_UP, KeyEvent.VK_DOWN,
                KeyEvent.VK_RIGHT, KeyEvent.VK_LEFT, KeyEvent.VK_SPACE,
                framePrincipal.jLX1J1, framePrincipal.jLY1J1);

        // criando Jogador 2
        this.jog2 = new Jogador(this, "azul.png", "bomba1.png", 15, 9,
                KeyEvent.VK_W, KeyEvent.VK_S,
                KeyEvent.VK_D, KeyEvent.VK_A, KeyEvent.VK_1,
                framePrincipal.jLX1J2, framePrincipal.jLY1J2);


        // Disparando o timer para começar a atualização do Loop principal do Jogo
        // Todo o conteúdo do método actionPerformed dessa classe será executado
        // a cada 60 milisegundos.
        this.timer.start();

    }

    /**
     * Principal METODO do JOGO, pois e' aqui que fica a logica que fara':
     * - processamento da logica do jogo (morreu, disparou, etc..)
     * - atualizar as coordenadas dos elementos na tela para o proximo redesenho
     * - redesenhar os elementos na tela
     * 
     * Este método é invocado a cada N milisegundos pelo Timer
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        this.framePrincipal.teclado.poll(); // processa os eventos no teclado
        this.processarLogica(); // atualiza as coordenadas dos elementos
        this.renderizar(); // desenha os elementos no buffer de desenho
        this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao


        // As linhas abaixo só servem para depuração. Pode removê-las
        if (jog1 != null) {
            framePrincipal.jLX2J1.setText("" + (this.jog1.posX + 40));
            framePrincipal.jLY2J1.setText("" + (this.jog1.posY + 40));
            framePrincipal.jLColJ1.setText("" + this.jog1.matColuna);
            framePrincipal.jLLinJ1.setText("" + this.jog1.matLinha);
            framePrincipal.jLvida1.setText("x " + this.jog1.nuvida);
        }
        // As linhas abaixo só servem para depuração. Pode removê-las
        if (jog2 != null) {
            framePrincipal.jLX2J2.setText("" + (this.jog2.posX + 40));
            framePrincipal.jLY2J2.setText("" + (this.jog2.posY + 40));
            framePrincipal.jLColJ2.setText("" + this.jog2.matColuna);
            framePrincipal.jLLinJ2.setText("" + this.jog2.matLinha);
            framePrincipal.jLvida2.setText("x " + this.jog2.nuvida);
        }
    }

    /**
     * Atualiza as coordenadas/estados de todos os elementos presentes no jogo
     * No caso, somente 2 jogadores, mas aqui pode-se colocar qualquer outro
     * elemento que venha a ser incluido no jogo
     */
    public void processarLogica() {
        //Chama o atualizar dos elementos
        if (this.jog1 != null) {
            this.jog1.atualizar();
        }
        if (this.jog2 != null) {
            this.jog2.atualizar();
        }
    }

    /**
     * Renderiza no buffer secundário todos os elementos já com suas coordenadas/estados
     * atualizados.
     */
    public void renderizar() {
        Graphics g = getBufferStrategy().getDrawGraphics();
        Graphics2D g2 = (Graphics2D) g;
        //Limpando a tela

        // Gerando um gradiente que ficara' como cor de fundo
        //GradientPaint gp = new GradientPaint(25, 25, Color.white,50, 50, Color.RED, true);
        //g2.setPaint(gp);

        // ou pintando com uma cor definida por RGB
        Color c = new Color(255, 210, 174);
        g2.setColor(c);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // desenhar cenario de fundo
        this.renderizarCenario(g2);

        // desenhando os elementos nao nulos
        if (this.jog1 != null) {
            this.jog1.desenhar(g2);
        }
        if (this.jog2 != null) {
            this.jog2.desenhar(g2);
        }
        /**
         * Veficica se algum usuario perdeu o jogo
         */
        if (this.jog1.nuvida == 0) {
            if (this.jog2.nuvida == 0) {
                short[][] cenario1 = {
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 1, 2, 1, 2, 1, 2, 1, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 2, 1, 2, 2, 1, 2, 2, 1, 2, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 1, 2, 1, 2, 1, 2, 1, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 1, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
                };
                this.cenario = cenario1;
                g2 = (Graphics2D) g;
                c = new Color(255, 210, 174);
                g2.setColor(c);
                g2.fillRect(0, 0, getWidth(), getHeight());
                this.renderizarCenario(g2);
                this.timer.stop();
                this.cenario = this.getCenario();
            } else {
                short[][] cenario1 = {
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 1, 2, 1, 2, 1, 2, 2, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 2, 1, 2, 2, 1, 2, 2, 2, 2, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 1, 2, 1, 2, 1, 2, 2, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
                };
                this.cenario = cenario1;
                g2 = (Graphics2D) g;
                c = new Color(255, 210, 174);
                g2.setColor(c);
                g2.fillRect(0, 0, getWidth(), getHeight());
                this.renderizarCenario(g2);
                this.timer.stop();
                this.cenario = this.getCenario();
            }
        } else {
            if (this.jog2.nuvida == 0) {
                short[][] cenario1 = {
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 2, 2, 1, 2, 1, 2, 1, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 2, 2, 2, 2, 1, 2, 2, 1, 2, 1, 2, 2, 0},
                    {0, 2, 0, 1, 0, 2, 2, 1, 2, 1, 2, 1, 0, 1, 0, 2, 0},
                    {0, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 2, 2, 0},
                    {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
                    {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
                };
                this.cenario = cenario1;
                g2 = (Graphics2D) g;
                c = new Color(255, 210, 174);
                g2.setColor(c);
                g2.fillRect(0, 0, getWidth(), getHeight());
                this.renderizarCenario(g2);
                this.timer.stop();
                this.cenario = this.getCenario();
            }
        }



        //liberando os contextos gráficos
        g.dispose();
        g2.dispose();

    }

    /**
     * Efetuando a troca das areas de desenho e apresentacao. 
     * No metodo redenrizar os elemtos graficos foram desenhados em um buffer
     * secundario, agora iremos fazer com que esse buffer secundario torne-se
     * primario, ou seja, que apareca de fato na tela
     */
    public void desenhar() {
        if (!getBufferStrategy().contentsLost()) {
            getBufferStrategy().show();
        }
    }

    /**
     * Fim de jogo/rodada
     */
    public void finalizar() {
        this.jog1 = null;
        this.jog2 = null;
        this.timer.stop();
        this.timer = null;
    }

    /**
     * Desenha o cenário com as paredes de tijolos e de ferro
     * @param g contexto gráfico
     */
    public void renderizarCenario(Graphics2D g) {
        for (int i = 0; i < 11; i++) { // linhas
            for (int j = 0; j < 17; j++) { // colunas
                if (this.cenario[i][j] != 2) { // se não for vazio, desenhe
                    g.drawImage(this.imgcenarios[this.cenario[i][j]], this.larguraBloco * j, this.larguraBloco * i, this.framePrincipal);

                }
            }
        }
    }

    /**
     * Verifica se na posicao lin,col da matriz tem um espaço vazio
     * @param lin numero da linha
     * @param col numero da coluna
     * @return true se estiver vazio, false caso contrario
     */
    public boolean estaLivre(int lin, int col) {

        return cenario[lin][col] == 2; // se for == 2, entao ira' retornar true, false caso contrario
    }
/**
     * 
     * @param lin
     * @param col
     * @return Verifica se é skill nivel da bomba
     */
    public boolean setbomba(int lin, int col) {

        return cenario[lin][col] == 3; // se for == 3, entao ira' retornar true, false caso contrario
    }
/**
     * 
     * @param lin
     * @param col
     * @return Verifica se é skill de boma
     */
    public boolean setnubomba(int lin, int col) {

        return cenario[lin][col] == 4; // se for == 4, entao ira' retornar true, false caso contrario
    }
/**
     * 
     * @param lin
     * @param col
     * @return Verifica se é skill de vida
     */
    public boolean setvida(int lin, int col) {

        return cenario[lin][col] == 5; // se for == 4, entao ira' retornar true, false caso contrario
    }

    /**
     * Indica em que posicao na tela encontrase o inicio dessa o retangulo que 
     * compreende uma celula
     * @param celula posicao na matriz que deseja-se descobrir
     * @return Coordenada na tela
     */
    public int celulaParaPixel(int celula) {
        return larguraBloco * celula;

    }

    /**
     * Indica qual celula o pixel esta' contido na matriz
     * @param pixel
     * @return 
     */
    public int pixelParaCelula(int pixel) {
        return pixel / larguraBloco;
    }
/**
     * 
     * @param a passa coordenada X da bomba a ser verificada
     * @param b passa coordenada X da bomba a ser verificada
     */
    public void verificaTEx(int a, int b) {
        int iX = a;
        int iY = b;
        int x1, x2;
        tempobomba = 200;

        this.arraybomba = jog1.arraybomba;
        for (int i = 0; i < arraybomba.size(); i++) {
            this.bombateste = arraybomba.get(i);
            if (iX >= this.bombateste.vetorX[0]) {
                if (iX <= this.bombateste.vetorX[1]) {
                    if (iY == this.bombateste.vetorY[2]) {
                        tempobomba = bombateste.tempoDeVidaDaBomba;

                    }
                }
            }
            if (iY >= this.bombateste.vetorY[0]) {
                if (iY <= this.bombateste.vetorY[1]) {
                    if (iX == this.bombateste.vetorX[2]) {
                        tempobomba = bombateste.tempoDeVidaDaBomba;

                    }
                }
            }
        }
        this.arraybomba = jog2.arraybomba;
        for (int i = 0; i < arraybomba.size(); i++) {
            this.bombateste = arraybomba.get(i);
            if (iX >= this.bombateste.vetorX[0]) {
                if (iX <= this.bombateste.vetorX[1]) {
                    if (iY == this.bombateste.vetorY[2]) {
                        tempobomba = bombateste.tempoDeVidaDaBomba;

                    }
                }
            }
            if (iY >= this.bombateste.vetorY[0]) {
                if (iY <= this.bombateste.vetorY[1]) {
                    if (iX == this.bombateste.vetorX[2]) {
                        tempobomba = bombateste.tempoDeVidaDaBomba;

                    }
                }
            }
        }
    }
/**
     * 
     * @return retorna tempo da ultima bomba verificada
     */
    public int getTempobomba() {
        return tempobomba;
    }
/**
     * 
     * @return Sorteia um numero relacionado ao Skill
     */
    public int getSkill() {
        int numero = 2 + (int) (Math.random() * 5);
        if (numero == 3) {
            if (NSb < 5) {
                return numero;
            }
            NSb++;
        }
        if (numero == 4) {
            if (NSv < 5) {
                return numero;
            }
            NSv++;
        }
        if (numero == 5) {
            if (NSvi < 5) {
                return numero;
            }
            NSvi++;
        }
        return (2);
    }
}